import { Button } from "@/components/ui/button";
import { SelectDMN } from "@/components/SelectDMN";
import { Input } from "@/components/ui/input";
import { Label } from "@/components/ui/label";
import { useCallback, useState } from "react";
import {
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
} from "@/components/ui/table";
import { ShieldX } from "lucide-react";
import { Alert, AlertDescription, AlertTitle } from "@/components/ui/alert";
import { cn } from "@/lib/utils";

export function Index() {
  const [currentDMNName, setCurrentDMNName] = useState("");
  const [errorMessage, setErrorMessage] = useState("");
  const setErrorMessageCallback = useCallback(async (errorMessage: string) => {
    setErrorMessage(errorMessage);

    await new Promise((resovle) => {
      setTimeout(resovle, 5000);
    });

    setErrorMessage("");
  }, []);

  const [currentResult, setCurrentResult] = useState({});

  return (
    currentDMNName == ""
      ? (
        <main className="h-screen w-screen flex flex-col justify-center items-center">
          <h1 className="mb-6">Choissisez votre DMN à évaluer</h1>

          <SelectDMN setCurrentDMNName={setCurrentDMNName} />
        </main>
      )
      : Object.keys(currentResult).length == 0
      ? (
        <main className="h-screen w-screen flex flex-col justify-center items-center">
          <Alert
            variant="destructive"
            className={cn(
              "fixed top-6 left-6 w-[500px]",
              errorMessage.length == 0 && "hidden",
            )}
          >
            <ShieldX className="h-5 w-5" />
            <AlertTitle>Une erreur est survenue...</AlertTitle>
            <AlertDescription>
              {errorMessage}
            </AlertDescription>
          </Alert>
          <h1 className="font-bold mb-6">
            Choisissez votre fichier JSON pour évaluer le DMN "{currentDMNName}"
          </h1>
          <div className="grid w-full max-w-sm items-center gap-1.5 mb-6">
            <Label htmlFor="JSON">JSON</Label>
            <Input
              onChange={async () => {
                const file = document.getElementById("JSON")!.files[0];
                if (file.name.split(".").pop() != "json") {
                  setErrorMessageCallback("Le fichier n'est pas de type JSON");

                  return;
                }

                let data: Object;

                try {
                  data = JSON.parse(await file.text());
                } catch (error: unknown) {
                  if (error instanceof SyntaxError) {
                    setErrorMessageCallback(
                      "Votre fichier comporte des erreures de syntaxes",
                    );

                    return;
                  }

                  // return "Something unexpected happened during the conversion of your data";
                }

                const response = await fetch(
                  `http://localhost:8080/${currentDMNName}`,
                  {
                    method: "post",
                    headers: {
                      "Accept": "application/json",
                      "Content-Type": "application/json",
                    },
                    body: JSON.stringify(data),
                  },
                );

                if (!response.ok) {
                  setErrorMessageCallback(
                    `Votre fichier ne contient pas les données nécessaires à l'évaluation du DMN "${currentDMNName}"`,
                  );

                  return;
                }

                const result = await response.json();

                const filteredResult = Object.keys(result).filter((key) => {
                  return !Object.keys(data).includes(key);
                }).reduce((object, key) => {
                  object[key] = result[key];
                  return object;
                }, {});

                setCurrentResult(filteredResult);
              }}
              id="JSON"
              type="file"
            />
          </div>
          <Button
            onClick={() => {
              setCurrentDMNName("");
            }}
          >
            Choisir son DMN
          </Button>
        </main>
      )
      : (
        <main className="h-screen w-screen flex flex-col justify-center items-center">
          <div className="mb-6">
            <Table className="max-w-[80%]">
              <TableHeader>
                <TableRow>
                  {Object.keys(currentResult).map((key) => {
                    return (
                      <TableHead>
                        {key}
                      </TableHead>
                    );
                  })}
                </TableRow>
              </TableHeader>
              <TableBody>
                <TableRow>
                  {Object.values(currentResult).map((value: any) => {
                    return (
                      <TableCell>
                        {value}
                      </TableCell>
                    );
                  })}
                </TableRow>
              </TableBody>
            </Table>
          </div>
          <Button
            onClick={() => {
              setCurrentDMNName("");
              setCurrentResult({});
            }}
          >
            Retour au début
          </Button>
        </main>
      )
  );
}
